import logging
import os
import csv
import json
from mongoengine import connect
from mongoengine import NotUniqueError
from mongoengine import OperationError
from mongoengine import DoesNotExist
from pymongo import MongoClient

from mykatlas.utils import check_args
from ga4ghmongo.schema import VariantCallSet
from ga4ghmongo.schema import VariantSet
from ga4ghmongo.schema import Variant
from ga4ghmongo.schema import VariantCall

from mykatlas._vcf import VCF
GLOBAL_VARIANT_SET_NAME = "global_atlas"

"""Adds variants to the database"""

logger = logging.getLogger(__name__)
client = MongoClient()
import redis

r = redis.StrictRedis()


class AtlasGenotypeResult(object):

    def __init__(self, sample, method, data, force=False):
        self.sample = sample
        self.method = method
        self.data = data
        self.call_set = None
        self.force = force

    def add(self):
        self._create_call_sets()
        self._create_calls()

        bitmap = self._create_genotype_bitmap(self.data["genotypes"])
        self._insert_bitmap(bitmap, name="gt")
        bitmap = self._create_filtered_bitmap(self.data["filtered"])
        self._insert_bitmap(bitmap, name="filtered")

    def _insert_bitmap(self, bitmap, name=""):
        pipe = r.pipeline()
        for i, j in enumerate(bitmap):
            pipe.setbit("_".join([self.call_set_name, name]), i, j)
        pipe.execute()

    def _create_var_list(self, sorted_calls):
        key = "var_hash"  # "_".join([self.call_set_name , "var_hash"])
        variants = [call[0][:64] for call in sorted_calls]
        pipe = r.pipeline()
        pipe.delete(key)
        for i in variants:
            pipe.rpush(key, i)
        pipe.execute()
        return variants

    def _create_genotype_bitmap(self, sorted_calls):
        bitmap = [int(call > 1) for call in sorted_calls]
        return bitmap

    def _create_filtered_bitmap(self, sorted_calls):
        # bitmap = [int(call[1]["info"]["conf"] > 1) for call in sorted_calls]
        return sorted_calls

    @property
    def call_set_name(self):
        return "_".join([self.sample, self.method])

    def _create_call_sets(self):
        try:
            self.call_set = VariantCallSet.create_and_save(
                name=self.call_set_name,
                variant_sets=[self.global_variant_set],
                sample_id=self.sample,
                info={
                    "variant_caller": self.method})
        except NotUniqueError:
            if self.force:
                # logger.info("There is already a call set for sample %s with method %s but add has been called with --force so this callset is being removed from the DB" %
                #     (self.sample, self.method))
                cs = VariantCallSet.objects.get(name="_".join([
                    self.sample,
                    self.method]))
                VariantCall.objects(call_set=cs).delete()
                cs.delete()
                self._create_call_sets()
            else:
                raise ValueError(
                    "There is already a call set for sample %s with method %s " %
                    (self.sample, self.method))

    def _create_calls(self):
        calls = []
        for var_hash, call in self.data["variant_calls"].items():
            if sum(call["genotype"]) >0 :
                var_hash = var_hash[:64]
                v = Variant.objects.get(var_hash=var_hash)
                c = VariantCall.create(
                    variant=v,
                    call_set=self.call_set,
                    genotype=call["genotype"],
                    genotype_likelihoods=call["genotype_likelihoods"],
                    info=call["info"])
                calls.append(c)
        VariantCall.objects.insert(calls)

    @property
    def global_variant_set(self):
        return VariantSet.objects.get(name=GLOBAL_VARIANT_SET_NAME)


def run(parser, args):
    args = parser.parse_args()
    args = check_args(args)
    if args.quiet:
        logger.setLevel(logging.ERROR)
    else:
        logger.setLevel(logging.INFO)
    DBNAME = 'atlas-%s' % (args.db_name)
    db = client[DBNAME]
    connect(DBNAME)
    logger.debug("Using DB %s" % DBNAME)

    for f in args.jsons:
        with open(f, 'r') as inf:
            data = json.load(inf)
        for sample, calls in data.items():
            AtlasGenotypeResult(sample, "atlas", calls, force=args.force).add()



from mongoengine import Document
from mongoengine import StringField
from mongoengine import DateTimeField
from mongoengine import IntField
from mongoengine import ReferenceField
from mongoengine import ListField
from mongoengine import FloatField
from mongoengine import DictField
from mongoengine import GenericReferenceField
from mongoengine import BooleanField
from mongoengine import queryset_manager
import datetime
from ga4ghmongo.schema.models.base import CreateAndSaveMixin


class AnalysisResult(Document, CreateAndSaveMixin):

    meta = {'allow_inheritance': True}

    files = ListField(StringField())
    created_at = DateTimeField(default=datetime.datetime.now)
    version = DictField(required=True)

    @classmethod
    def create(cls):
        raise NotImplementedError("Implemented in subclass")



#! /usr/bin/env python
from __future__ import print_function
import sys
import os


import os
import argparse
import sys
sys.path.append(
    os.path.realpath(
        os.path.join(
            os.path.dirname(__file__),
            "..")))
from mykatlas.version import __version__
import logging

from base import ArgumentParserWithDefaults
from base import DEFAULT_DB_NAME
from base import sequence_parser_mixin
from base import sequence_or_binary_parser_mixin
from base import probe_set_mixin
from base import force_mixin
from base import genotyping_mixin

DEFAULT_KMER_SIZE = os.environ.get("KMER_SIZE", 31)


def run_subtool(parser, args):
    if args.command == 'add':
        from mykatlas.cmds.add import run
    elif args.command == "add-gt":
        from mykatlas.cmds.atlasadd import run
    elif args.command == "dump-probes":
        from mykatlas.cmds.dump import run
    elif args.command == "make-probes":
        from mykatlas.cmds.makeprobes import run
    elif args.command == "genotype":
        from mykatlas.cmds.genotype import run
    elif args.command == "walk":
        from mykatlas.cmds.walk import run
    elif args.command == "place":
        from mykatlas.cmds.place import run
    elif args.command == "diff":
        from mykatlas.cmds.diff import run
    # run the chosen submodule.
    run(parser, args)


def main():
    #########################################
    # create the top-level parser
    #########################################
    parser = argparse.ArgumentParser(
        prog='atlas',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("--version", help="atlas version",
                        action="version",
                        version="%(prog)s " + str(__version__))
    subparsers = parser.add_subparsers(
        title='[sub-commands]',
        dest='command',
        parser_class=ArgumentParserWithDefaults)

    db_parser_mixin = argparse.ArgumentParser(add_help=False)
    db_parser_mixin.add_argument(
        '--db_name',
        metavar='db_name',
        type=str,
        help='db_name',
        default="default")

    ##########
    # Add
    ##########
    parser_add = subparsers.add_parser(
        'add',
        help='adds a set of variants to the atlas',
        parents=[db_parser_mixin, force_mixin])
    parser_add.add_argument('vcf', type=str, help='a vcf file')
    parser_add.add_argument('reference_set', type=str, help='reference set')
    parser_add.add_argument(
        '-m',
        '--method',
        type=str,
        help='variant caller method (e.g. CORTEX)',
        default="NotSpecified")
    parser_add.set_defaults(func=run_subtool)

    parser_add_gt = subparsers.add_parser(
        'add-gt',
        help='adds a set of atlas genotype calls to the atlas',
        parents=[db_parser_mixin, force_mixin])
    parser_add_gt.add_argument('jsons', type=str, nargs='+',
                               help='json output from `atlas genotype`')
    parser_add_gt.add_argument(
        '-m',
        '--method',
        type=str,
        help='variant caller method (e.g. CORTEX)',
        default="atlas")
    parser_add_gt.set_defaults(func=run_subtool)

    # ##########
    # # Dump panel
    # ##########
    parser_dump = subparsers.add_parser(
        'dump-probes',
        help='dump a probe set of variant alleles',
        parents=[db_parser_mixin, force_mixin])
    parser_dump.add_argument(
        'reference_filepath',
        metavar='reference_filepath',
        type=str,
        help='reference_filepath')
    parser_dump.add_argument(
        '--kmer',
        metavar='kmer',
        type=int,
        help='kmer length',
        default=31)
    parser_dump.add_argument(
        '-v',
        '--verbose',
        default=False,
        action="store_true")
    parser_dump.set_defaults(func=run_subtool)

    ##################
    ### Make Probes ##
    ##################

    parser_make_probes = subparsers.add_parser(
        'make-probes', help='make probes from a list of variants',
        parents=[db_parser_mixin])
    parser_make_probes.add_argument(
        'reference_filepath',
        metavar='reference_filepath',
        type=str,
        help='reference_filepath')
    parser_make_probes.add_argument(
        '-f',
        '--vcf',
        type=str,
        help='Use variants defined in a VCF file',
        default=[])
    parser_make_probes.add_argument(
        '-v',
        '--variant',
        type=str,
        action='append',
        help='Variant in DNA positions e.g. A1234T',
        default=[])
    parser_make_probes.add_argument(
        '-t',
        '--text_file',
        type=str,
        help='Text file containing variants as rows A1234T')
    parser_make_probes.add_argument(
        '-g',
        '--genbank',
        type=str,
        help='Genbank file containing genes as features')
    parser_make_probes.add_argument(
        '-k',
        '--kmer',
        type=int,
        help='kmer length',
        default=31)
    parser_make_probes.add_argument(
        '--no-backgrounds',
        help='Build probe set against reference only ignoring nearby variants',
        default=False,
        action="store_true")
    parser_make_probes.set_defaults(func=run_subtool)

    # ##########
    # # Genotype
    # ##########
    parser_geno = subparsers.add_parser(
        'genotype',
        parents=[
            sequence_or_binary_parser_mixin,
            probe_set_mixin,
            force_mixin,
            genotyping_mixin],
        help='genotype a sample using a probe set')
    parser_geno.set_defaults(func=run_subtool)

    ##############
    ## Walk ##
    #############
    parser_walk = subparsers.add_parser(
        'walk',
        parents=[
            sequence_or_binary_parser_mixin,
            probe_set_mixin,
            force_mixin,
            genotyping_mixin],
        help='Walk through a graph using an existing sequence probe set as seeds. default walking algorithm is a depth first search')
    parser_walk.add_argument(
        '--also-genotype',
        default=False,
        action="store_true")
    parser_walk.add_argument('--show-all-paths', action="store_true")
    parser_walk.set_defaults(func=run_subtool)

    ##############
    ## Place ##
    #############
    parser_place = subparsers.add_parser(
        'place', help='Place a sample on a prebuilt tree', parents=[
            db_parser_mixin, force_mixin])
    parser_place.add_argument(
        'sample',
        metavar='sample',
        type=str,
        help='sample id')
    parser_place.add_argument('--tree', metavar='tree', type=str, help='tree')
    parser_place.add_argument(
        '--searchable_samples',
        metavar='searchable_samples',
        type=str,
        help='list of samples (file)')
    parser_place.add_argument(
        '--no-cache',
        default=False,
        action="store_true")
    parser_place.set_defaults(func=run_subtool)

    ##

    ##############
    ## Place ##
    #############
    parser_diff = subparsers.add_parser(
        'diff',
        help='Outputs novel sequence by calculating the difference between the sequence and combined graph',
        parents=[sequence_or_binary_parser_mixin])
    parser_diff.add_argument(
        'graph',
        metavar='graph',
        type=str,
        help='The graph to compare new sample against')
    parser_diff.add_argument(
        '--add',
        default=False,
        action="store_true",
        help="after comparing, add the new sample to the graph")
    parser_diff.set_defaults(func=run_subtool)
    ##

    args = parser.parse_args()
    args.func(parser, args)


if __name__ == "__main__":
    main()



from __future__ import print_function
import os
import json
import csv
import glob
import logging
import subprocess
from copy import copy

from mykatlas.typing import SequenceProbeCoverage
from mykatlas.typing import VariantProbeCoverage
from mykatlas.typing import ProbeCoverage
from mykatlas.typing import Panel

from mykatlas.typing.typer.presence import GeneCollectionTyper
from mykatlas.typing.typer.variant import VariantTyper
from mykatlas.typing.typer.base import DEFAULT_MINOR_FREQ
from mykatlas.typing.typer.base import DEFAULT_ERROR_RATE

from ga4ghmongo.schema import VariantCallSet
from ga4ghmongo.schema import Variant

from mykatlas.cortex import McCortexGenoRunner

from mykatlas.utils import get_params
from mykatlas.utils import split_var_name
from mykatlas.utils import median

logger = logging.getLogger(__name__)


class CoverageParser(object):

    def __init__(
            self,
            sample,
            panel_file_paths,
            kmer,
            force,
            seq=None,
            ctx=None,
            threads=2,
            memory="1GB",
            panels=None,
            verbose=True,
            tmp_dir='/tmp/',
            skeleton_dir='atlas/data/skeletons/',
            mccortex31_path="mccortex31"):
        self.sample = sample
        self.seq = seq
        self.ctx = ctx
        self.kmer = kmer
        self.force = force
        self.covgs = {"variant": {}, "presence": {}}
        self.variant_covgs = self.covgs["variant"]
        self.gene_presence_covgs = self.covgs["presence"]
        self.mc_cortex_runner = None
        self.verbose = verbose
        self.skeleton_dir = skeleton_dir
        self.tmp_dir = tmp_dir
        self.panel_file_paths = panel_file_paths
        self.panels = []
        self.mccortex31_path = mccortex31_path
        self.threads = threads
        self.memory = memory
        for panel_file_path in self.panel_file_paths:
            panel = Panel(panel_file_path)
            self.panels.append(panel)
        if self.seq and self.ctx:
            raise ValueError("Can't have both -1 and -c")

    def run(self):
        self._run_cortex()
        self._parse_covgs()

    def _run_cortex(self):
        self.mc_cortex_runner = McCortexGenoRunner(
            sample=self.sample,
            panels=self.panels,
            seq=self.seq,
            ctx=self.ctx,
            kmer=self.kmer,
            force=self.force,
            threads=self.threads,
            memory=self.memory,
            panel_name=self.panel_name,
            tmp_dir=self.tmp_dir,
            skeleton_dir=self.skeleton_dir,
            mccortex31_path=self.mccortex31_path)
        self.mc_cortex_runner.run()

    def estimate_depth(self):
        depth = []
        for variant_coverages in self.variant_covgs.values():
            for variant_covg in variant_coverages:
                if variant_covg.reference_coverage.median_depth > 0:
                    depth.append(variant_covg.reference_coverage.median_depth)
        for spcs in self.gene_presence_covgs.values():
            __median_depth = median(
                [spc.median_depth for spc in spcs.values()])
            if __median_depth > 0:
                depth.append(__median_depth)
        _median = median(depth)
        if _median < 1:
            return 1
        else:
            return _median

    def remove_temporary_files(self):
        self.mc_cortex_runner.remove_temporary_files()

    @property
    def panel_name(self):
        return "-".join([panel.name for panel in self.panels])

    def _parse_summary_covgs_row(self, row):
        try:
            return row[0], int(row[2]), int(row[3]), 100 * float(row[4])
        except ValueError:
            logger.warning("Failed to parse %s" % str(row))
            return row[0], 0, 0, 0.0

    def _parse_covgs(self):
        with open(self.mc_cortex_runner.covg_tmp_file_path, 'r') as infile:
            self.reader = csv.reader(infile, delimiter="\t")
            for row in self.reader:
                allele, median_depth, min_depth, percent_coverage = self._parse_summary_covgs_row(
                    row)
                allele_name = allele.split('?')[0]
                if self._is_variant_panel(allele_name):
                    self._parse_variant_panel(row)
                else:
                    self._parse_seq_panel(row)

    def _is_variant_panel(self, allele_name):
        try:
            alt_or_ref, _id = allele_name.split('-')
            return bool(alt_or_ref)
        except ValueError:
            return False

    def _parse_seq_panel(self, row):
        allele, median_depth, min_depth, percent_coverage = self._parse_summary_covgs_row(
            row)
        probe_coverage = ProbeCoverage(
            percent_coverage=percent_coverage,
            median_depth=median_depth,
            min_depth=min_depth)

        allele_name = allele.split('?')[0]
        params = get_params(allele)
        panel_type = params.get("panel_type", "presence")
        name = params.get('name')
        version = params.get(
            'version',
            '1')
        if panel_type in ["variant", "presence"]:
            sequence_probe_coverage = SequenceProbeCoverage(
                name=name,
                probe_coverage=probe_coverage,
                version=version,
                length=params.get("length"))
            try:
                self.covgs[panel_type][name][version] = sequence_probe_coverage
            except KeyError:
                self.covgs[panel_type][name] = {}
                self.covgs[panel_type][name][version] = sequence_probe_coverage

        else:
            # Species panels are treated differently
            l = int(params.get("length", -1))
            try:
                self.covgs[panel_type][name]["total_bases"] += l
                if percent_coverage > 75 and median_depth > 0:
                    self.covgs[panel_type][name][
                        "percent_coverage"].append(percent_coverage)
                    self.covgs[panel_type][name]["length"].append(l)
                    self.covgs[panel_type][name]["median"].append(median_depth)
            except KeyError:
                if panel_type not in self.covgs:
                    self.covgs[panel_type] = {}
                self.covgs[panel_type][name] = {}
                self.covgs[panel_type][name]["total_bases"] = l
                if percent_coverage > 75 and median_depth > 0:
                    self.covgs[panel_type][name][
                        "percent_coverage"] = [percent_coverage]
                    self.covgs[panel_type][name]["length"] = [l]
                    self.covgs[panel_type][name]["median"] = [median_depth]
                else:
                    self.covgs[panel_type][name]["percent_coverage"] = []
                    self.covgs[panel_type][name]["length"] = []
                    self.covgs[panel_type][name]["median"] = []

    def _parse_variant_panel(self, row):
        allele, reference_median_depth, min_depth, reference_percent_coverage = self._parse_summary_covgs_row(
            row)
        var_name = allele.split('?')[0].split('-')[1]
        params = get_params(allele)
        num_alts = int(params.get("num_alts", 0))
        reference_coverage = ProbeCoverage(
            percent_coverage=reference_percent_coverage,
            median_depth=reference_median_depth,
            min_depth=min_depth)
        alternate_coverages = []
        for i in range(num_alts):
            row = self.reader.next()
            alt_allele, alternate_median_depth, min_depth, alternate_percent_coverage = self._parse_summary_covgs_row(
                row)
            alternate_coverages.append(
                ProbeCoverage(
                    min_depth=min_depth,
                    percent_coverage=alternate_percent_coverage,
                    median_depth=alternate_median_depth))
        variant_probe_coverage = VariantProbeCoverage(
            reference_coverage=reference_coverage,
            alternate_coverages=alternate_coverages,
            var_name=var_name,
            params=params)
        try:
            self.variant_covgs[allele].append(variant_probe_coverage)
        except KeyError:
            self.variant_covgs[allele] = [variant_probe_coverage]


class Genotyper(object):

    """Takes output of mccortex coverages and types"""

    def __init__(
            self,
            sample,
            expected_depths,
            variant_covgs,
            gene_presence_covgs,
            contamination_depths=[],
            base_json={},
            report_all_calls=False,
            ignore_filtered=False,
            expected_error_rate=DEFAULT_ERROR_RATE,
            minor_freq=DEFAULT_MINOR_FREQ,
            variant_confidence_threshold=0,
            sequence_confidence_threshold=0,
            min_gene_percent_covg_threshold=100):
        self.sample = sample
        self.variant_covgs = variant_covgs
        self.gene_presence_covgs = gene_presence_covgs
        self.out_json = base_json
        if expected_depths < 1:
            self.expected_depths = 1
        else:
            self.expected_depths = expected_depths
        self.contamination_depths = contamination_depths
        self.variant_calls = {}
        self.sequence_calls = {}
        self.variant_calls_dict = {}
        self.sequence_calls_dict = {}
        self.expected_error_rate = expected_error_rate
        self.report_all_calls = report_all_calls
        self.ignore_filtered = ignore_filtered
        self.minor_freq = minor_freq
        self.variant_confidence_threshold = variant_confidence_threshold
        self.sequence_confidence_threshold = sequence_confidence_threshold
        self.min_gene_percent_covg_threshold = min_gene_percent_covg_threshold

    def run(self):
        self._type()

    def _type(self):
        self._type_genes()
        self._type_variants()

    def _type_genes(self):
        gt = GeneCollectionTyper(
            expected_depths=self.expected_depths,
            contamination_depths=self.contamination_depths,
            confidence_threshold=self.sequence_confidence_threshold)
        for gene_name, gene_collection in self.gene_presence_covgs.items():
            self.gene_presence_covgs[gene_name] = gt.type(
                gene_collection,
                min_gene_percent_covg_threshold=self.min_gene_percent_covg_threshold)
            self.sequence_calls_dict[gene_name] = [
                gpc.to_mongo().to_dict() for gpc in self.gene_presence_covgs[gene_name]]
        self.out_json[self.sample][
            "sequence_calls"] = self.sequence_calls_dict

    def _type_variants(self):
        self.out_json[self.sample]["variant_calls"] = {}
        gt = VariantTyper(
            expected_depths=self.expected_depths,
            error_rate=self.expected_error_rate,
            contamination_depths=self.contamination_depths,
            ignore_filtered=self.ignore_filtered,
            minor_freq=self.minor_freq,
            confidence_threshold=self.variant_confidence_threshold)
        genotypes = []
        filters = []
        for probe_name, probe_coverages in self.variant_covgs.items():
            # variant = self._create_variant(probe_name)
            variant = None
            call = gt.type(probe_coverages, variant=variant)
            genotypes.append(sum(call["genotype"]))
            filters.append(int(call["info"]["filter"] == "PASS"))
            if sum(call["genotype"]) > 0 or not call[
                    "genotype"] or self.report_all_calls:
                self.variant_calls[probe_name] = call
                probe_id = probe_name.split("?")[0].split("-")[1]
                self.variant_calls_dict[
                    probe_id] = call
        self.out_json[self.sample]["genotypes"] = genotypes
        self.out_json[self.sample]["filtered"] = filters
        self.out_json[self.sample]["variant_calls"] = self.variant_calls_dict

    def _create_variant(self, probe_name):
        names = []
        params = get_params(probe_name)
        if params.get("mut"):
            names.append("_".join([params.get("gene"), params.get("mut")]))
        var_name = probe_name.split('?')[0].split('-')[1]
        names.append(var_name)
        try:
            # If it's a variant panel we can create a variant
            ref, start, alt = split_var_name(var_name)
            return Variant.create(
                start=start,
                reference_bases=ref,
                alternate_bases=[alt],
                names=names,
                info=params)
        except AttributeError:
            return None



from __future__ import print_function

import os
import subprocess
import urllib2
import logging
import tempfile
logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)


class McCortexRunner(object):

    def __init__(self, mccortex31_path):
        self.mccortex31_path = mccortex31_path


class McCortexJoin(McCortexRunner):

    def __init__(
            self,
            sample,
            intersect_graph,
            ingraph,
            mccortex31_path="mccortex31"):
        super(McCortexJoin, self).__init__(mccortex31_path)
        self.sample = sample
        self.intersect_graph = intersect_graph
        self.ingraph = ingraph
        self.out_ctx_dir = tempfile.mkdtemp()
        self.out_ctx_path = os.path.join(
            self.out_ctx_dir,
            "%s_int.ctx" %
            sample)

    def run(self):
        self._run_cortex()
        return self.out_ctx_path

    def _run_cortex(self):
        cmd = [self.mccortex31_path,
               "join",
               "-q",
               "--out", self.out_ctx_path,
               "--intersect",
               self.intersect_graph,
               self.ingraph]
        subprocess.check_output(cmd)


class McCortexUnitigs(McCortexRunner):

    def __init__(self, ingraph, mccortex31_path="mccortex31"):
        super(McCortexUnitigs, self).__init__(mccortex31_path)
        self.ingraph = ingraph

    def run(self):
        return self._run_cortex()

    def _run_cortex(self):
        cmd = [self.mccortex31_path,
               "unitigs",
               "-q",
               self.ingraph]
        return subprocess.check_output(cmd)


class McCortexSubgraph(McCortexRunner):

    def __init__(
            self,
            sample,
            rmgraph,
            ingraph,
            tmp_dir=None,
            mccortex31_path="mccortex31"):
        super(McCortexSubgraph, self).__init__(mccortex31_path=mccortex31_path)
        self.rmgraph = rmgraph
        self.sample = sample
        self.ingraph = ingraph
        if tmp_dir is None:
            self.out_ctx_dir = tempfile.mkdtemp()
        else:
            self.out_ctx_dir = tmp_dir
        self.out_ctx_path = os.path.join(
            self.out_ctx_dir,
            "%s_new.ctx" %
            sample)

    def run(self):
        self._run_cortex()
        return self.out_ctx_path

    def _run_cortex(self):
        cmd = [self.mccortex31_path,
               "view",
               "-q",
               "-k",
               self.rmgraph,
               "|",
               "awk",
               "'{print $1}'",
               "|",
               self.mccortex31_path,
               "subgraph",
               "-q",
               "--out",
               self.out_ctx_path,
               "--invert",
               "--seq",
               "-",
               self.ingraph
               ]
        logger.debug(subprocess.list2cmdline(cmd))
        logger.debug(" ".join(cmd))
        cmd = subprocess.Popen(" ".join(cmd), shell=True).wait()


class McCortexGenoRunner(McCortexRunner):

    def __init__(
            self,
            sample,
            panels,
            seq=None,
            ctx=None,
            kmer=31,
            threads=2,
            memory="1GB",
            force=False,
            panel_name=None,
            tmp_dir='/tmp/',
            skeleton_dir='data/skeletons/',
            mccortex31_path="mccortex31"):
        super(McCortexRunner, self).__init__()
        self.sample = sample
        self.panels = panels
        self.seq = seq
        self.ctx = ctx
        self.kmer = kmer
        self.force = force
        self._panel_name = panel_name
        self.tmp_dir = tmp_dir
        self.threads = threads
        self.memory = memory
        if skeleton_dir == 'atlas/data/skeletons/':
            skeleton_dir = os.path.realpath(
                os.path.join(
                    os.path.dirname(
                        os.path.realpath(__file__)),
                    "..",
                    skeleton_dir))
        self.skeleton_dir = skeleton_dir
        self.mccortex31_path = mccortex31_path
        if self.seq and self.ctx:
            raise ValueError("Can't have both -1 and -c")

    def run(self):
        if self.force or not os.path.exists(self.covg_tmp_file_path):
            self._check_panels()
            self._run_cortex()

    def _check_panels(self):
        # If panel does not exists then build it
        for panel in self.panels:
            if not os.path.exists(panel.filepath):
                raise ValueError(
                    "Could not find a panel at %s." %
                    panel.filepath)

    def _run_cortex(self):
        # If ctx binary does not exist then build it
        self._build_panel_binary_if_required()
        # Now get coverage on panel
        self._run_coverage_if_required()

    def _build_panel_binary_if_required(self):
        if not os.path.exists(self.ctx_skeleton_filepath) or self.force:
            if os.path.exists(self.ctx_skeleton_filepath):
                os.remove(self.ctx_skeleton_filepath)
            # panel
            seq_list = self._create_sequence_list()
            cmd = [self.mccortex31_path,
                   "build",
                   "-m %s" % self.memory,
                   "-t", "%i" % self.threads,
                   "-k",
                   str(self.kmer)] + seq_list + [self.ctx_skeleton_filepath]
            # print (cmd)
            subprocess.check_output(cmd)

    def _create_sequence_list(self):
        seq_list = []
        seq_list.extend(["-s", "%s" % self.panel_name[:100]])
        for panel in self.panels:
            seq_list.extend(["-1", panel.filepath])
        return seq_list

    def _run_coverage_if_required(self):
        if not os.path.exists(
                self.ctx_tmp_filepath) or not os.path.exists(
                self.covg_tmp_file_path) or self.force:
            if os.path.exists(self.ctx_tmp_filepath):
                os.remove(self.ctx_tmp_filepath)
            if os.path.exists(self.covg_tmp_file_path):
                os.remove(self.covg_tmp_file_path)
            logger.debug("running %s" % " ".join(self.coverages_cmd))
            subprocess.check_output(self.coverages_cmd)
        else:
            # print "Warning: Using pre-built binaries. Run with --force if
            # panel has been updated."
            pass

    @property
    def coverages_cmd(self):
        if self.seq:
            return self.coverages_cmd_seq
        elif self.ctx:
            return self.coverages_cmd_ctx
        else:
            raise ValueError("Need either seq or ctx binary to run coverages")

    @property
    def base_geno_command(self):
        return [self.mccortex31_path, "geno", "-t", "%i" % self.threads,
                "-m %s" % self.memory,
                "-k", str(self.kmer),
                "-o", self.covg_tmp_file_path]

    @property
    def coverages_cmd_seq(self):
        cmd = self.base_geno_command
        cmd.extend(["-I", self.ctx_skeleton_filepath])
        cmd.extend(["-s", self.sample_name])
        for seq in self.seq:
            cmd.extend(["-1", seq])
        for panel in self.panels:
            cmd.extend(["-c", panel.filepath])
        cmd.append(self.ctx_tmp_filepath)
        return cmd

    @property
    def coverages_cmd_ctx(self):
        cmd = self.base_geno_command
        cmd.extend(["-g", self.ctx])
        for panel in self.panels:
            cmd.extend(["-c", panel.filepath])
        cmd.append(self.ctx_tmp_filepath)
        return cmd

    @property
    def sample_name(self):
        return "-".join([self.sample, str(self.kmer)])

    @property
    def panel_name(self):
        if self._panel_name is None:
            self._panel_name = "-".join([p.name.replace('/', '-')
                                         for p in self.panels])
        return self._panel_name

    @property
    def sample_panel_name(self):
        return "_".join([self.sample_name, self.panel_name])

    @property
    def ctx_tmp_filepath(self):
        return "%s/%s.ctx" % (self.tmp_dir, self.sample_panel_name)

    @property
    def covg_tmp_file_path(self):
        return "%s/%s.covgs" % (self.tmp_dir, self.sample_panel_name)

    @property
    def ctx_skeleton_filepath(self):
        return os.path.abspath(
            "%s/%s_%i.ctx" %
            (self.skeleton_dir, self.panel_name.replace(
                "/",
                "-")[
                :100],
                self.kmer))

    def remove_temporary_files(self):
        os.remove(self.ctx_tmp_filepath)
        os.remove(self.covg_tmp_file_path)
